#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-09 09:53:18.676780

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.system_interfaces.general_statistics_setting_att import GeneralStatisticsSettingAtt


class ServerStatisticsSettingAtt(GeneralStatisticsSettingAtt):

    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-09 09:53:18.676780)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     System.SettingController
                |                         System.GeneralStatisticsSettingAtt
                |                             ServerStatisticsSettingAtt
                | 
                | Interface for Server statistic Controller
                | Role: the server statistics controller manages the values of all or only a part
                | of the attributes available for the thematic.
                | 
                | For the definitions of methods and variables common to every thematic, see the
                | GeneralStatisticsSettingAtt
                | 
                | Copyright © 1999-2011, Dassault Systèmes. All rights reserved.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.server_statistics_setting_att = com_object

    def __repr__(self):
        return f'ServerStatisticsSettingAtt(name="{ self.name }")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.hybrid_shape_interfaces.hybrid_shape_circle import HybridShapeCircle
from pycatia.in_interfaces.reference import Reference


class HybridShapeCircle3Points(HybridShapeCircle):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.HybridShape
                |                         CATGSMIDLItf.HybridShapeCircle
                |                             HybridShapeCircle3Points
                | 
                | Represents the hybrid shape circle object defined using three
                | points.
                | Role: To access the data of the hybrid shape circle object.
                | 
                | This data includes the circle three passing points.
                | 
                | Use the CATIAHybridShapeFactory to create a HybridShapeCircle2PointsRad
                | object.
                | 
                | See also:
                |     HybridShapeFactory
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.hybrid_shape_circle3_points = com_object

    @property
    def element1(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Element1() As Reference
                | 
                |     Returns or sets the circle first passing point.
                |     Sub-element(s) supported (see Boundary object): Vertex.
                | 
                |     Example:
                |         This example retrieves the first passing point of the HybShpCircle3Pt
                |         hybrid shape circle in HybShpCircle3PtFirstPassingPoint
                |         point.
                | 
                |          Dim HybShpCircle3PtFirstPassingPoint As Reference
                |          Set HybShpCircle3PtFirstPassingPoint=
                |          HybShpCircle3Pt.Element1

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_circle3_points.Element1)

    @element1.setter
    def element1(self, reference_element: Reference):
        """
        :param Reference reference_element:
        """

        self.hybrid_shape_circle3_points.Element1 = reference_element.com_object

    @property
    def element2(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Element2() As Reference
                | 
                |     Returns or sets the circle second passing point.
                |     Sub-element(s) supported (see Boundary object): Vertex.
                | 
                |     Example:
                |         This example sets the second passing point of the HybShpCircle3Pt
                |         hybrid shape circle as the Point2 point.
                | 
                |          HybShpCircle3Pt.Element2 Point2

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_circle3_points.Element2)

    @element2.setter
    def element2(self, reference_element: Reference):
        """
        :param Reference reference_element:
        """

        self.hybrid_shape_circle3_points.Element2 = reference_element.com_object

    @property
    def element3(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Element3() As Reference
                | 
                |     Returns or sets the circle third passing point.
                |     Sub-element(s) supported (see Boundary object): Vertex.
                | 
                |     Example:
                |         This example retrieves the third passing point of the HybShpCircle3Pt
                |         hybrid shape circle in HybShpCircle3PtThirdPassingPoint
                |         point.
                | 
                |          Dim HybShpCircle3PtThirdPassingPoint As Reference
                |          Set HybShpCircle3PtThirdPassingPoint=
                |          HybShpCircle3Pt.Element3

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_circle3_points.Element3)

    @element3.setter
    def element3(self, reference_element: Reference):
        """
        :param Reference reference_element:
        """

        self.hybrid_shape_circle3_points.Element3 = reference_element.com_object

    @property
    def support(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Support() As Reference
                | 
                |     Returns or sets the circle support surface.
                |     Sub-element(s) supported (see Boundary object): Face.
                | 
                |     Example:
                |         This example retrieves in HybShpCircleSupportSurf the support surface
                |         of the HybShpCircle hybrid shape circle.
                | 
                |          Dim HybShpCircleSupportSurf As Reference 
                |          HybShpCircleSupportSurf = HybShpCircle.Support

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_circle3_points.Support)

    @support.setter
    def support(self, reference_support: Reference):
        """
        :param Reference reference_support:
        """

        self.hybrid_shape_circle3_points.Support = reference_support.com_object

    def remove_support(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub RemoveSupport()
                | 
                |     Removes the support surface.

        :return: None
        :rtype: None
        """
        return self.hybrid_shape_circle3_points.RemoveSupport()

    def __repr__(self):
        return f'HybridShapeCircle3Points(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from pycatia.sketcher_interfaces.point_2D import Point2D
from pycatia.system_interfaces.system_service import SystemService


class ControlPoint2D(Point2D):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     SketcherInterfaces.GeometricElement
                |                         SketcherInterfaces.Geometry2D
                |                             SketcherInterfaces.Point2D
                |                                 ControlPoint2D
                | 
                | Class defining a spline control point in 2D Space.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.control_point_2d = com_object

    @property
    def curvature(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Curvature() As double
                | 
                |     Returns the curvature properties of the spline control
                |     point
                | 
                |     Parameters:
                | 
                |         oCurvature
                |             The curvature of the tangent determined at the control
                |             point

        :return: float
        :rtype: float
        """

        return self.control_point_2d.Curvature

    @curvature.setter
    def curvature(self, value: float):
        """
        :param float value:
        """

        self.control_point_2d.Curvature = value

    def get_tangent(self) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetTangent(CATSafeArrayVariant oTangent)
                | 
                |     Returns the tangent properties of the spline control point
                | 
                |     Parameters:
                | 
                |         oTangent[0]
                |             The X Coordinate of the tangent determined at the control point
                |             
                |         oTangent[1]
                |             The Y Coordinate of the tangent determined at the control
                |             point

        :return: tuple
        :rtype: tuple
        """

        vba_function_name = 'get_tangent'
        vba_code = """
        Public Function get_tangent(control_point2_d)
            Dim oTangent (1)
            control_point2_d.GetTangent oTangent
            get_tangent = oTangent
        End Function
        """

        system_service = self.application.system_service
        return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_tangent(self, i_tangent_x: float, i_tangent_y: float) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetTangent(double iTangentX,
                | double iTangentY)
                | 
                |     Imposes the tangent properties of the spline control point
                | 
                |     Parameters:
                | 
                |         iTangentX
                |             The X Coordinate of the tangent determined at the control point
                |             
                |         iTangentY
                |             The Y Coordinate of the tangent determined at the control
                |             point

        :param float i_tangent_x:
        :param float i_tangent_y:
        :return: None
        :rtype: None
        """
        return self.control_point_2d.SetTangent(i_tangent_x, i_tangent_y)

    def unset_curvature(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub UnsetCurvature()
                | 
                |     Unsets the curvature properties of the spline control point

        :return: None
        :rtype: None
        """
        return self.control_point_2d.UnsetCurvature()

    def unset_tangent(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub UnsetTangent()
                | 
                |     Unsets the tangent properties of the spline control point

        :return: None
        :rtype: None
        """
        return self.control_point_2d.UnsetTangent()

    def __repr__(self):
        return f'ControlPoint2D(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.knowledge_interfaces.parameter import Parameter


class IntParam(Parameter):

    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     KnowledgeInterfaces.Parameter
                |                         IntParam
                | 
                | Represents the integer parameter.
                | The following example shows how to create it:
                | 
                |  Dim CATDocs As Documents
                |  Set CATDocs = CATIA.Documents
                |  Dim part1 As Dccument
                |  Set part1   = CATDocs.Add("CATPart") 
                |  Dim  year As IntParam
                |  Set year    = part1.Part.Parameters.CreateInteger("year", 1998)
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.int_param = com_object

    @property
    def range_max(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property RangeMax() As long
                | 
                |     Returns or sets the value of the upper bound that the parameter object
                |     value can take.
                | 
                |     Example:
                |         This example sets the RangeMax value to 0 if its value is smaller than
                |         0:
                | 
                |          If (Length.RangeMax < 0.0 and Length.RangeMaxValidity <> 0) 
                |          Then
                |              Length.RangeMax = 0.0
                |          End If

        :return: int
        :rtype: int
        """

        return self.int_param.RangeMax

    @range_max.setter
    def range_max(self, value: int):
        """
        :param int value:
        """

        self.int_param.RangeMax = value

    @property
    def range_max_validity(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property RangeMaxValidity() As long
                | 
                |     Returns or sets the type of the upper bound of the
                |     parameter.
                | 
                |     0
                |         the upper bound is meaningless 
                |     1
                |         the upper bound can be reached 
                |     2
                |         the upper bound cannot be reached

        :return: int
        :rtype: int
        """

        return self.int_param.RangeMaxValidity

    @range_max_validity.setter
    def range_max_validity(self, value: int):
        """
        :param int value:
        """

        self.int_param.RangeMaxValidity = value

    @property
    def range_min(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property RangeMin() As long
                | 
                |     Returns or sets the value of the lower bound that the parameter object
                |     value can take.
                | 
                |     Example:
                |         This example sets the RangeMin value to 0 if its value is bigger than
                |         0:
                | 
                |          If (Length.RangeMin > 0.0 and Length.RangeMinValidity <> 0) 
                |          Then
                |              Length.RangeMin = 0.0
                |          End If

        :return: int
        :rtype: int
        """

        return self.int_param.RangeMin

    @range_min.setter
    def range_min(self, value: int):
        """
        :param int value:
        """

        self.int_param.RangeMin = value

    @property
    def range_min_validity(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property RangeMinValidity() As long
                | 
                |     Returns or sets the type of the lower bound of the
                |     parameter.
                | 
                |     0
                |         the lower bound is meaningless 
                |     1
                |         the lower bound can be reached 
                |     2
                |         the lower bound cannot be reached

        :return: int
        :rtype: int
        """

        return self.int_param.RangeMinValidity

    @range_min_validity.setter
    def range_min_validity(self, value: int):
        """
        :param int value:
        """

        self.int_param.RangeMinValidity = value

    @property
    def value(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Value() As long
                | 
                |     Returns or sets the value of the integer parameter. Units are expressed in
                |     the IS unit system.
                | 
                |     Example:
                |         This example sets the year value to 0 if its value is equal to
                |         2000:
                | 
                |          If (year.Value = 2000)  Then
                |              year.Value = 0
                |          End If

        :return: int
        :rtype: int
        """

        return self.int_param.Value

    @value.setter
    def value(self, value: int):
        """
        :param int value:
        """

        self.int_param.Value = value

    def get_enumerate_values(self, o_safe_array: tuple) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub GetEnumerateValues(CATSafeArrayVariant oSafeArray)
                | 
                |     Returns an array containing the different values that the int param can
                |     take in the case of multiple values.
                | 
                |     Example:
                | 
                |          Dim enumValues () as Variant
                |          ReDim enumValues (anIntegerParameter.GetEnumerateValuesSize() -
                |          1)
                |          anIntegerParameter.GetEnumerateValues(enumValues)
                |          For i = LBound(enumValues) to UBound(enumValues)
                |            ...
                |          Next

        :param tuple o_safe_array:
        :return: None
        :rtype: None
        """
        return self.int_param.GetEnumerateValues(o_safe_array)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_enumerate_values'
        # # vba_code = """
        # # Public Function get_enumerate_values(int_param)
        # #     Dim oSafeArray (2)
        # #     int_param.GetEnumerateValues oSafeArray
        # #     get_enumerate_values = oSafeArray
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_enumerate_values_size(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func GetEnumerateValuesSize() As long
                | 
                |     Returns the number of enumerate values.

        :return: int
        :rtype: int
        """
        return self.int_param.GetEnumerateValuesSize()

    def set_enumerate_values(self, i_safe_array: tuple) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub SetEnumerateValues(CATSafeArrayVariant iSafeArray)
                | 
                |     Sets an array containing the different values that the real param can take
                |     in the case of multiple values.

        :param tuple i_safe_array:
        :return: None
        :rtype: None
        """
        return self.int_param.SetEnumerateValues(i_safe_array)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_enumerate_values'
        # # vba_code = """
        # # Public Function set_enumerate_values(int_param)
        # #     Dim iSafeArray (2)
        # #     int_param.SetEnumerateValues iSafeArray
        # #     set_enumerate_values = iSafeArray
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def suppress_enumerate_values(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub SuppressEnumerateValues()
                | 
                |     Resets the status of the object to a single value object.

        :return: None
        :rtype: None
        """
        return self.int_param.SuppressEnumerateValues()

    def __repr__(self):
        return f'IntParam(name="{ self.name }")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-10 10:58:07.270911

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.

"""
from typing import Iterator
from typing import TYPE_CHECKING

from pycatia.base_interfaces.pycatia import PyCATIA
from pycatia.system_interfaces.any_object import AnyObject

# from pycatia.system_interfaces.cat_base_dispatch import CATBaseDispatch

if TYPE_CHECKING:
    from pycatia.in_interfaces.application import Application


class Collection(PyCATIA):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-10 10:58:07.270911)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 Collection
                |
                | Represents the base object for collections.
                | As a base object, it provides properties and methods shared by any other
                | object.

    """

    def __init__(self, com_object, child_object=AnyObject):
        super().__init__()
        self.com_object = com_object
        self.child_object = child_object

    @property
    def application(self) -> 'Application':
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-10 10:58:07.270911)
                | o Property Application() As Application (Read Only)
                |
                |     Returns the application. The application is the root object in the object
                |     structure and can be retrieved from any object in the object structure using
                |     the Application property. The Application property is the way to jump from any
                |     object up to the root of the object data structure, allowing then to navigate
                |     downwards. For in-process scripting, the application is always referred to as
                |     CATIA. Note that the Application property of the Application object returns the
                |     Application object itself.
                |
                |     Example:
                |         This example retrieves in CurrentApplication the application object,
                |         root of the object structure, from a given object of this structure: a document
                |         refered to using the MyDocCollecion variable.
                |
                |          Dim CurrentApplication As Application
                |          Set CurrentApplication = MyDocCollecion.Application

        :return: Application
        :rtype: Application
        """
        from pycatia.in_interfaces.application import Application
        return Application(self.com_object.Application)

    @property
    def count(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-10 10:58:07.270911)
                | o Property Count() As long (Read Only)
                |
                |     Returns the number of objects in the collection. This is handy to scan all
                |     the objects in a collection.
                |
                |     Example:
                |         This example retrieves in ObjectNumber the number of objects currently
                |         gathered in MyCollection.
                |
                |          ObjectNumber = MyCollection.Count

        :return: int
        :rtype: int
        """

        return self.com_object.Count

    @property
    def name(self) -> str:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-10 10:58:07.270911)
                | o Property Name() As CATBSTR (Read Only)
                |
                |     Returns or sets the name of the object. The name is a character string you
                |     can assign to any object to handle it easier. In the case of an object part of
                |     a collection, the name can often be used in place of the object rank to
                |     retrieve or remove the object, providing the Item and Remove methods of the
                |     collection feature an argument with the Variant type. If the object has no name
                |     set, the name returned is the one of its parent.
                |
                |     Example:
                |         This example sets to MyObject the name Nice and Handy Object
                |         Name.
                |
                |          MyObject.Name("Nice and Handy Object Name")

        :return: str
        :rtype: str
        """

        return self.com_object.Name

    @property
    def parent(self) -> AnyObject:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-10 10:58:07.270911)
                | o Property Parent() As CATBaseDispatch (Read Only)
                |
                |     Returns the parent object. The parent object of a given object is the
                |     object that created this object, usually the object just above in the object
                |     tree structure and that aggregates it. In the case of an object part of a
                |     collection, the parent object is not the collection object itself, but the
                |     object that aggregates the collection object. The Parent property is the way to
                |     step upwards in the object data structure. Note that the Parent property of the
                |     Application object returns the Application object itself.
                |
                |     Example:
                |         This example retrieves in ParentObject the parent object of the
                |         GivenObject object.
                |
                |          Dim ParentObject As AnyObject
                |          Set ParentObject = GivenObject.Parent

        :return: AnyObject
        :rtype: AnyObject
        """

        return AnyObject(self.com_object.Parent)

    def get_item(self, id_name: str) -> AnyObject:
        """

        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-10 10:58:07.270911))
                | o Func GetItem(CATBSTR IDName) As CATBaseDispatch
                |
                |     Returns an object from its name.
                |     Role: To retrieve an object when only its name is available. You should not
                |     use this method, but you can find it in the macros generated by the
                |     Tools->Macro command.
                |
                |     Parameters:
                |
                |         IDName
                |             The searched object name
                |
                |     Returns:
                |         The searched object

        :param str id_name:
        :return: AnyObject
        :rtype: AnyObject
        """

        return self.child_object(self.com_object.GetItem(id_name))

    def get_item_by_index(self, index):
        """
        :param str/int index: relation name or index
        :return: child_object
        """

        return self.child_object(self.com_object.Item(index))

    def get_item_names(self):
        names = []
        for i in range(self.com_object.Count):
            name = self.com_object.Item(i + 1).Name
            names.append(name)

        return names

    def get_item_by_name(self, name):
        for i in range(self.com_object.Count):
            if self.com_object.Item(i + 1).Name == name:
                return self.child_object(self.com_object.Item(i + 1))

        return None

    def items(self):
        """
        :return: [self.child_object()]
        """
        items_list = []

        for i in range(self.com_object.Count):
            item = self.child_object(self.com_object.Item(i + 1))
            items_list.append(item)

        return items_list

    def __len__(self):

        return self.count

    def __getitem__(self, n: int) -> AnyObject:
        if (n + 1) > self.count:
            raise StopIteration

        return AnyObject(self.com_object.item(n + 1))

    def __iter__(self) -> Iterator[AnyObject]:
        for i in range(self.count):
            yield self.child_object(self.com_object.item(i + 1))

    def __repr__(self):
        return f'Collection(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.drafting_interfaces.drawing_sheet import DrawingSheet
from pycatia.drafting_interfaces.drawing_document import DrawingDocument


class DrawingRoot(DrawingDocument):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     DrawingRoot
                | 
                | Represents the drawing object in drawing documents.
                | 
                | Warning: This interface is not available with 2D Layout for 3D
                | Design.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.drawing_root_com = com_object

    @property
    def active_sheet(self) -> DrawingSheet:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property ActiveSheet() As DrawingSheet
                | 
                |     Retrieves or sets the active sheet of the drawing.
                |     Warning: This method is not available with 2D Layout for 3D
                |     Design.
                | 
                |     Example:
                |         This example retrieves the active sheet in the drawing of the active
                |         document, supposed to be a drawing document.
                | 
                |          CATIA.ActiveDocument.DrawingRoot.GetActiveSheet

        :return: DrawingSheet
        :rtype: DrawingSheet
        """

        return DrawingSheet(self.drawing_root_com.ActiveSheet)

    @active_sheet.setter
    def active_sheet(self, value: DrawingSheet):
        """
        :param DrawingSheet value:
        """

        self.drawing_root_com.ActiveSheet = value

    def reorder_sheets(self, i_ordered_sheets: tuple) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub reorder_Sheets(CATSafeArrayVariant iOrderedSheets)
                | 
                |     Changes the positions of the sheets in this drawing according to the given
                |     ordered list. iOrderedSheets is the result of a permutation applied to the list
                |     of all the sheets of this drawing, with the following constraint: For every
                |     non-detail sheet, there is not any detail sheet appearing before in
                |     iOrderedSheets.
                | 
                |     Example:
                |         This example inverts the sheet order of a drawing made of exactly two
                |         regular sheets.
                | 
                |          Set drwsheets = CATIA.ActiveDocument.Sheets
                |          Set drwsheetsorder =  CATIA.ActiveDocument.DrawingRoot
                |          Set sheet1 = drwsheets.item(1)
                |          Set sheet2 = drwsheets.item(2)
                |          newsheetorder = Array(sheet2, sheet1)
                |          drwsheetsorder.reorder_Sheets(newsheetorder)

        :param tuple i_ordered_sheets:
        :return: None
        :rtype: None
        """
        sheets = [x.com_object for x in i_ordered_sheets]
        return self.drawing_root_com.reorder_Sheets(sheets)

    def __repr__(self):
        return f'DrawingRoot(name="{self.name}")'



#! /user/bin/python3.6


def create_measurable(spa_workbench, com_reference):
    """

    :param spa_workbench:
    :param com_reference:
    :return:
    """

    return spa_workbench.GetMeasurable(com_reference)



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.in_interfaces.reference import Reference
from pycatia.in_interfaces.references import References
from pycatia.part_interfaces.dress_up_shape import DressUpShape


class AutoDraft(DressUpShape):

    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.Shape
                |                         PartInterfaces.DressUpShape
                |                             AutoDraft
                | 
                | Represents the AutoDraft shape.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.auto_draft = com_object

    @property
    def functional_face(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property FunctionalFace(Reference iFace) (Write Only)

        :return: None
        :rtype: None
        """

        return None

    @functional_face.setter
    def functional_face(self, reference: Reference):
        """
        :param Reference reference:
        """

        self.auto_draft.FunctionalFace = reference

    @property
    def functional_faces(self) -> References:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property FunctionalFaces() As References (Read Only)
                | 
                |     Returns or sets the functional faces.
                | 
                |     Example:
                |         The following example returns in FunctionalFaces the list functional
                |         faces of the AutoDraft AutoDraft, and then sets NewFunctionalFace as a
                |         functional face:
                | 
                |          Set FunctionalFaces = AutoDraft.FunctionalFace
                |          AutoDraft.FunctionalFace = NewFunctionalFace

        :return: References
        :rtype: References
        """

        return References(self.auto_draft.FunctionalFaces)

    @property
    def main_draft_angle(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property MainDraftAngle() As Reference
                | 
                |     Returns or sets the main draft angle.
                | 
                |     Example:
                |         The following example returns in MainDraftAngle the main draft angle of
                |         the AutoDraft AutoDraft, and then sets it to
                |         NewMainDraftAngle.:
                | 
                |          Set MainDraftAngle = AutoDraft.MainDraftAngle
                |          AutoDraft.MainDraftAngle = NewMainDraftAngle

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.auto_draft.MainDraftAngle)

    @main_draft_angle.setter
    def main_draft_angle(self, value: Reference):
        """
        :param Reference value:
        """

        self.auto_draft.MainDraftAngle = value

    @property
    def mode(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Mode() As Reference
                | 
                |     Returns or sets the draft mode.
                | 
                |     Example:
                |         The following example returns in Mode the mode of the draft AutoDraft
                |         AutoDraft, and then sets it to NewMode:
                | 
                |          Set Mode = AutoDraft.Mode
                |          AutoDraft.Mode = NewMode

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.auto_draft.Mode)

    @mode.setter
    def mode(self, value: Reference):
        """
        :param Reference value:
        """

        self.auto_draft.Mode = value

    @property
    def parting_element(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property PartingElement() As Reference
                | 
                |     Returns or sets the parting element.
                | 
                |     Example:
                |         The following example returns in PartingElement the parting element of
                |         the AutoDraft AutoDraft, and then sets it to
                |         NewpartingElement:
                | 
                |          Set PartingElement = AutoDraft.PartingElement
                |          AutoDraft.PartingElement = NewPartingElement

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.auto_draft.PartingElement)

    @parting_element.setter
    def parting_element(self, value: Reference):
        """
        :param Reference value:
        """

        self.auto_draft.PartingElement = value

    @property
    def pulling_direction(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property PullingDirection() As Reference
                | 
                |     Returns or sets the pulling direction.
                | 
                |     Example:
                |         The following example returns in PullingDirection the pulling direction
                |         of the AutoDraft AutoDraft, and then sets it to
                |         NewPullingDirection.:
                | 
                |          Set PullingDirection = AutoDraft.PullingDirection
                |          AutoDraft.PullingDirection = NewPullingDirection

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.auto_draft.PullingDirection)

    @pulling_direction.setter
    def pulling_direction(self, value: Reference):
        """
        :param Reference value:
        """

        self.auto_draft.PullingDirection = value

    def __repr__(self):
        return f'AutoDraft(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from typing import Iterator

from pycatia.navigator_interfaces.hyperlink import Hyperlink
from pycatia.system_interfaces.any_object import AnyObject
from pycatia.system_interfaces.collection import Collection
from pycatia.types.general import cat_variant


class Hyperlinks(Collection):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.Collection
                |                     Hyperlinks

    
    """

    def __init__(self, com_object):
        super().__init__(com_object, child_object=Hyperlink)
        self.hyperlinks = com_object

    def add(self, i_object: AnyObject) -> Hyperlink:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func Add(AnyObject iObject) As Hyperlink
                | 
                |     Adds a Hyperlink to the Hyperlinks collection. Be careful: only one
                |     Hyperlink instance is allowed per object which supports the hyperlink. If one
                |     adds a Hyperlink on an object which already has a Hyperlink, the call fails.
                |     The Hyperlink name is internally generated. If one wants to manage Hyperlink
                |     using its name, he has to set it manually.
                | 
                |     Parameters:
                | 
                |         iObject
                |             The object which supports the hyperlink. 
                | 
                |     Example:
                | 
                |              The following example adds a Hyperlink
                |             
                | 
                |             Dim NewHyperlink As Hyperlink
                |             Set NewHyperlink = TheHyperlinks.Add(iObject)

        :param AnyObject i_object:
        :return: Hyperlink
        :rtype: Hyperlink
        """
        return Hyperlink(self.hyperlinks.Add(i_object.com_object))

    def get_hyperlink(self, i_object: AnyObject) -> Hyperlink:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func GetHyperlink(AnyObject iObject) As Hyperlink
                | 
                |     Retrieve an Hyperlink associated to an object
                | 
                |     Parameters:
                | 
                |         iObject
                |             The object which supports the hyperlink. The call fails otherwise.
                |             
                | 
                |     Returns:
                |         The retrieved Hyperlink if it exists 
                |     Example:
                | 
                |              This example retrieves an existing one in the TheHyperlinks
                |              collection.
                |             
                | 
                |             Dim NewHyperlink As Hyperlink
                |             Set NewHyperlink = TheHyperlinks.GetHyperlink(iObject)

        :param AnyObject i_object:
        :return: Hyperlink
        :rtype: Hyperlink
        """
        return Hyperlink(self.hyperlinks.GetHyperlink(i_object.com_object))

    def item(self, i_index: cat_variant) -> Hyperlink:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func Item(CATVariant iIndex) As Hyperlink
                | 
                |     Returns an Hyperlink using its index from the Hyperlinks
                |     collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index or the name of the hyperlink to retrieve from the
                |             collection of Hyperlinks. As a numerics, this index is the rank of the
                |             Hyperlink in the collection. The index of the first Hyperlink in the collection
                |             is 1, and the index of the last Hyperlink is Count. As a string, it is the name
                |             you assigned to the Hyperlink. 
                | 
                |     Returns:
                |         The retrieved Hyperlink 
                |     Example:
                | 
                |              This example retrieves in ThisHyperlink the ninth Hyperlink
                |              ,
                |             and in ThatHyperlink the Hyperlink named
                |              Hyperlink3 from the TheHyperlinks collection. 
                |             
                | 
                |             Dim ThisHyperlink As Hyperlink
                |             Set ThisHyperlink = TheHyperlinks.Item(9)
                |             Dim ThatHyperlink As Hyperlink
                |             Set ThatHyperlink = TheHyperlinks.Item("Hyperlink3")

        :param cat_variant i_index:
        :return: Hyperlink
        :rtype: Hyperlink
        """
        return Hyperlink(self.hyperlinks.Item(i_index))

    def remove(self, i_index: cat_variant) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub Remove(CATVariant iIndex)
                | 
                |     Removes a Hyperlink from the Hyperlinks collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index or the name of the Hyperlink to retrieve from the
                |             collection of Hyperlinks. As a numerics, this index is the rank of the
                |             Hyperlink in the collection. The index of the first Hyperlink in the collection
                |             is 1, and the index of the last Hyperlink is Count. As a string, it is the name
                |             you assigned to the Hyperlink. 
                | 
                |     Example:
                | 
                |              The following example removes the tenth Hyperlink and the
                |              Hyperlink named
                |              Hyperlink 2 from the TheHyperlinks collection.
                |             
                | 
                |             TheHyperlinks.Remove(10)
                |             TheHyperlinks.Remove("Hyperlink2")

        :param cat_variant i_index:
        :return: None
        :rtype: None
        """
        return self.hyperlinks.Remove(i_index)

    def __getitem__(self, n: int) -> Hyperlink:
        if (n + 1) > self.count:
            raise StopIteration

        return Hyperlink(self.hyperlinks.item(n + 1))

    def __iter__(self) -> Iterator[Hyperlink]:
        for i in range(self.count):
            yield self.child_object(self.com_object.item(i + 1))

    def __repr__(self):
        return f'Hyperlinks(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.drafting_interfaces.drawing_leaders import DrawingLeaders
from pycatia.drafting_interfaces.drawing_text_properties import DrawingTextProperties
from pycatia.system_interfaces.any_object import AnyObject


class DrawingText(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     DrawingText
                | 
                | Represents a drawing text in a drawing view.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.drawing_text = com_object

    @property
    def anchor_position(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property AnchorPosition() As CatTextAnchorPosition
                | 
                |     Returns or sets the anchor position of the drawing text.
                | 
                |     Example:
                |         This example sets the anchor position of the MyText drawing text to top
                |         left position.
                | 
                |          MyText.AnchorPosition = TopLeft

        :return: int
        :rtype: int
        """

        return self.drawing_text.AnchorPosition

    @anchor_position.setter
    def anchor_position(self, value: int):
        """
        :param int value:
        """

        self.drawing_text.AnchorPosition = value

    @property
    def angle(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Angle() As double
                | 
                |     Returns or sets the angle of the drawing text. The angle is measured
                |     between the axis system of the drawing view and the local axis system of the
                |     drawing text. The angle is measured in radians and is counted
                |     counterclockwise.
                | 
                |     Example:
                |         This example sets the angle of the MyText drawing Text to 90 degrees
                |         clockwise. You first need to compute the angle in degrees and set the minus
                |         sign to indicate the rotation is clockwise.
                | 
                |          Angle90Clockwise = -90
                |          MyText.Angle = Angle90Clockwise

        :return: float
        :rtype: float
        """

        return self.drawing_text.Angle

    @angle.setter
    def angle(self, value: float):
        """
        :param float value:
        """

        self.drawing_text.Angle = value

    @property
    def associative_element(self) -> AnyObject:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property AssociativeElement() As CATBaseDispatch
                | 
                |     Returns or sets the associative object of the drawing
                |     text.
                | 
                |     Example:
                |         This example sets an associative line of the MyText drawing text to top
                |         left position.
                | 
                |          MyText.AssociativeElement = line

        :return: AnyObject
        :rtype: AnyObject
        """

        return AnyObject(self.drawing_text.AssociativeElement)

    @associative_element.setter
    def associative_element(self, value: AnyObject):
        """
        :param AnyObject value:
        """

        self.drawing_text.AssociativeElement = value

    @property
    def frame_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property FrameType() As CatTextFrameType
                | 
                |     Returns or sets the frame type of the drawing text.
                | 
                |     Example:
                |         This example sets the frame type of the MyText drawing text to an
                |         ellipse.
                | 
                |          MyText.FrameType = catEllipse

        :return: int
        :rtype: int
        """

        return self.drawing_text.FrameType

    @frame_type.setter
    def frame_type(self, value: int):
        """
        :param int value:
        """

        self.drawing_text.FrameType = value

    @property
    def leaders(self):
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Leaders() As DrawingLeaders (Read Only)
                | 
                |     Returns the drawing leader collection of the drawing text.
                | 
                |     Example:
                |         This example retrieves in LeaderCollection the collection of leaders of
                |         the MyText drawing text.
                | 
                |          Dim LeaderCollection As DrawingLeaders
                |          Set LeaderCollection = MyText.Leaders

        :return: DrawingLeaders
        :rtype: DrawingLeaders
        """

        return DrawingLeaders(self.drawing_text.Leaders)

    @property
    def text(self) -> str:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Text() As CATBSTR
                | 
                |     Returns or sets character string that makes up the text.
                | 
                |     Example:
                |         This example retrieves in CharString the character string of the MyText
                |         drawing text.
                | 
                |          CharString = MyText.Text

        :return: str
        :rtype: str
        """

        return self.drawing_text.Text

    @text.setter
    def text(self, value: str):
        """
        :param str value:
        """

        self.drawing_text.Text = value

    @property
    def text_properties(self) -> DrawingTextProperties:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property TextProperties() As DrawingTextProperties (Read
                | Only)
                | 
                |     Returns the text properties of the drawing text. Allows to modify the whole
                |     text properties. To manage a sub part of the text use
                |     GetParameterOnSubString
                | 
                |     Example:
                |         This example retrieves in TextProperties the text properties of the
                |         MyText drawing text.
                | 
                |          Dim TextProperties As DrawingTextProperties
                |          Set TextProperties = MyText.TextProperties

        :return: DrawingTextProperties
        :rtype: DrawingTextProperties
        """

        return DrawingTextProperties(self.drawing_text.TextProperties)

    @property
    def wrapping_width(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property WrappingWidth() As double
                | 
                |     Returns or sets the wrapping width of the drawing text.
                | 
                |     Example:
                |         This example sets the wrapping width of the MyText drawing text to
                |         50.
                | 
                |          MyText.WrappingWidth = 50.

        :return: float
        :rtype: float
        """

        return self.drawing_text.WrappingWidth

    @wrapping_width.setter
    def wrapping_width(self, value: float):
        """
        :param float value:
        """

        self.drawing_text.WrappingWidth = value

    @property
    def x(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property x() As double
                | 
                |     Returns or sets the x coordinate of the text. It is expressed with respect
                |     to the current view coordinate system. This coordinate, like any length, is
                |     measured in meters.
                | 
                |     Example:
                |         This example retrieves the x coordinate of the text MyText drawing
                |         text.
                | 
                |          X = MyText.x

        :return: float
        :rtype: float
        """

        return self.drawing_text.x

    @x.setter
    def x(self, value: float):
        """
        :param float value:
        """

        self.drawing_text.x = value

    @property
    def y(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property y() As double
                | 
                |     Returns or sets the y coordinate of the text. It is expressed with respect
                |     to the view coordinate system. This coordinate, like any length, is measured in
                |     meters.
                | 
                |     Example:
                |         This example sets the y coordinate of the text MyText drawing text to 5
                |         inches. You need first to convert the 5 inches into
                |         meters.
                | 
                |          NewYCoordinate = 5*25.4/1000
                |          MyText.y =  NewYCoordinate

        :return: float
        :rtype: float
        """

        return self.drawing_text.y

    @y.setter
    def y(self, value: float):
        """
        :param float value:
        """

        self.drawing_text.y = value

    def activate_frame(self, itype: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub ActivateFrame(CatTextFrameType itype)
                | 
                |     Activates the text frame of the drawing text.
                | 
                |     Example:
                |         This example adds a rectangle frame to MyText drawing
                |         text.
                | 
                |          CatTextFrameType ityp = catRectangle
                |          MyText.ActivateFrame(itype)
                |          
                | 
                |         This example removes the frame to MyText drawing text.
                | 
                |          CatTextFrameType ityp = catNone
                |          MyText.ActivateFrame(itype)

        :param int itype:
        :return: None
        :rtype: None
        """
        return self.drawing_text.ActivateFrame(itype)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'activate_frame'
        # # vba_code = """
        # # Public Function activate_frame(drawing_text)
        # #     Dim itype (2)
        # #     drawing_text.ActivateFrame itype
        # #     activate_frame = itype
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_font_name(self, i_first, inb_character):
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func GetFontName(long iFirst,
                | long inbCharacter) As CATBSTR
                | 
                |     Returns the font name on a substring of the drawing text.
                | 
                |     Parameters:
                | 
                |         iFirst
                |             The first character to which the property should apply
                |             
                |         inbCharacter
                |             The number of characters to which the property should apply
                |             
                | 
                |     Returns:
                |         oFontName The name of the font 
                |     Example:
                |         This example gets the MyText drawing text font.
                | 
                |          oFontName = MyText.GetFontName(0, 0)

        :param int i_first:
        :param int inb_character:
        :return: str
        :rtype: str
        """
        return self.drawing_text.GetFontName(i_first, inb_character)

    def get_font_size(self, i_first: int, inb_character: int) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func GetFontSize(long iFirst,
                | long inbCharacter) As double
                | 
                |     Returns the font size on a substring of the drawing text.
                | 
                |     Parameters:
                | 
                |         iFirst
                |             The first character to which the property should apply
                |             
                |         inbCharacter
                |             The number of characters to which the property should apply
                |             
                | 
                |     Returns:
                |         oFontSize The size of the font 
                |     Example:
                |         This example gets the MyText font size.
                | 
                |          oFontSize = MyText.GetFontSize(0, 0)

        :param int i_first:
        :param int inb_character:
        :return: float
        :rtype: float
        """
        return self.drawing_text.GetFontSize(i_first, inb_character)

    def get_modifiable_in_2d_component_instances(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func GetModifiableIn2DComponentInstances() As boolean
                | 
                |     Returns if the text is modifiable or not in 2D component instances. The
                |     text must own to a 2D component (NOT to a view)
                | 
                |     Example:
                |         This example retrieves if MyText drawing text is modifiable or
                |         not
                | 
                |          IsModifiable = MyText.GetModifiableIn2DComponentInstances

        :return: bool
        :rtype: bool
        """
        return self.drawing_text.GetModifiableIn2DComponentInstances()

    def get_parameter_on_sub_string(self, i_param: int, i_first: int, inb_character: int) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func GetParameterOnSubString(CatTextProperty iParam,
                | long iFirst,
                | long inbCharacter) As long
                | 
                |     Returns a property on a substring of the drawing text.
                | 
                |     Parameters:
                | 
                |         iParam
                |             The drawing text property 
                |         iFirst
                |             The first character to which the property should apply
                |             
                |         inbCharacter
                |             The number of characters to which the property should apply
                |             
                | 
                |     Returns:
                |         oval The value corresponding to the property 
                |     Example:
                |         This example gets the parameter Italic on MyText drawing
                |         text.
                | 
                |          CatTextProperty iParam = catItalic 
                |          iFirst = 0
                |          inbCharacter = 0
                |          oval = MyText.GetParameterOnsubString(iParam, iFirst, inbCharacter)

        :param int i_param:
        :param int i_first:
        :param int inb_character:
        :return: int
        :rtype: int
        """
        return self.drawing_text.GetParameterOnSubString(i_param, i_first, inb_character)

    def insert_variable(self, i_first: int, inb_character: int, ibase: AnyObject) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub InsertVariable(long iFirst,
                | long inbCharacter,
                | CATBaseDispatch ibase)
                | 
                |     Sets a Parameter in a string of the drawing text.
                | 
                |     Parameters:
                | 
                |         iFirst
                |             The first character from which the parameter is inserted
                |             
                |         inbCharacter
                |             The number of characters the parameter will replace
                |             
                |         iParameter
                |             The parameter to be inserted 
                |         Example:
                |             This example sets a parameter right at the end of MyText drawing
                |             text.
                | 
                |              Dim DrwDocument As DrawingDocument
                |              Set DrwDocument = CATIA.ActiveDocument
                | 
                |              Dim iParameter As Parameter
                |              Set iParameter = DrwDocument.Parameters.Item("Drawing/Sheet.1/ViewMakeUp.1/Scale")
                | 
                |              MyText.InsertVariable 0, 0, iParameter

        :param int i_first:
        :param int inb_character:
        :param AnyObject ibase:
        :return: None
        :rtype: None
        """
        return self.drawing_text.InsertVariable(i_first, inb_character, ibase.com_object)

    def set_font_name(self, i_first: int, inb_character: int, i_font_name: str) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub SetFontName(long iFirst,
                | long inbCharacter,
                | CATBSTR iFontName)
                | 
                |     Sets the font size on a substring of the drawing text.
                | 
                |     Parameters:
                | 
                |         iFirst
                |             The first character to which the property should apply
                |             
                |         inbCharacter
                |             The number of characters to which the property should apply
                |             
                |         iFontName
                |             The name of the font
                | 
                |             Example:
                |                 This example sets the MyText drawing text font as Courrier 10
                |                 BT.
                | 
                |                  MyText.SetFontName 0,  0, "Courrier 10 BT"

        :param int i_first:
        :param int inb_character:
        :param str i_font_name:
        :return: None
        :rtype: None
        """
        return self.drawing_text.SetFontName(i_first, inb_character, i_font_name)

    def set_font_size(self, i_first: int, inb_character: int, i_font_size: float) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub SetFontSize(long iFirst,
                | long inbCharacter,
                | double iFontSize)
                | 
                |     Sets the font size on a substring of the drawing text.
                | 
                |     Parameters:
                | 
                |         iFirst
                |             The first character to which the property should apply
                |             
                |         inbCharacter
                |             The number of characters to which the property should apply
                |             
                |         iFontSize
                |             The size of the font 
                |         Example:
                |             This example sets the MyText font size to 3.5.
                | 
                |              iFontSize = 3.5
                |              MyText.SetFontSize 0,  0, iFontSize

        :param int i_first:
        :param int inb_character:
        :param float i_font_size:
        :return: None
        :rtype: None
        """
        return self.drawing_text.SetFontSize(i_first, inb_character, i_font_size)

    def set_modifiable_in_2d_component_instances(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub SetModifiableIn2DComponentInstances()
                | 
                |     Sets the text as modifiable in 2D component instances.The text must own to
                |     a 2D component (NOT to a view).then ,its content will be modifiable inside
                |     instances of this 2D component.
                | 
                |     Example:
                |         This example sets the MyText drawing text as
                |         modifiable.
                | 
                |          MyText.SetModifiableIn2DComponentInstances

        :return: None
        :rtype: None
        """
        return self.drawing_text.SetModifiableIn2DComponentInstances()

    def set_parameter_on_sub_string(self, i_param: int, i_first: int, inb_character: int, i_val: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub SetParameterOnSubString(CatTextProperty iParam,
                | long iFirst,
                | long inbCharacter,
                | long iVal)
                | 
                |     Sets a property on a substring of the drawing text.
                | 
                |     Parameters:
                | 
                |         iParam
                |             The drawing text property 
                |         iFirst
                |             The first character to which the property should apply
                |             
                |         inbCharacter
                |             The number of characters to which the property should apply
                |             
                |         iVal
                |             The value to be applied according to the property 
                |         Example:
                |             This example sets all MyText drawing text in bold
                |             character.
                | 
                |              CatTextProperty iParam = catBold 
                |              iFirst = 0
                |              inbCharacter = 0
                |              ival = 1
                |              MyText.SetParameterOnsubString iParam, iFirst, inbCharacter,
                |              ival

        :param int i_param:
        :param int i_first:
        :param int inb_character:
        :param int i_val:
        :return: None
        :rtype: None
        """
        return self.drawing_text.SetParameterOnSubString(i_param, i_first, inb_character, i_val)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_parameter_on_sub_string'
        # # vba_code = """
        # # Public Function set_parameter_on_sub_string(drawing_text)
        # #     Dim iParam (2)
        # #     drawing_text.SetParameterOnSubString iParam
        # #     set_parameter_on_sub_string = iParam
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def __repr__(self):
        return f'DrawingText(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.system_interfaces.setting_controller import SettingController


class DmuTolSettingAtt(SettingController):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     System.SettingController
                |                         DMUTolSettingAtt
                | 
                | Represents the DMU Tolerancing setting controller object.
                | Role: The DMU Tolerancing setting controller object deals with the setting
                | parameters displayed in:
                | 
                |     The DMU Tolerancing property page for the Related Surface Color and the
                |     Design Mode setting parameters.
                |     To access this property page:
                |         Click the Options command in the Tools menu
                |         Click + left of Digital Mockup to unfold the workbench
                |         list
                |         Click DMU Tolerancing Review
                |     The Cgr Management property page for the Save in CGR setting
                |     parameter.
                |     To access this property page:
                |         Click the Options command in the Tools menu
                |         Click + left of Infrastructure to unfold the workbench
                |         list
                |         Click Product Structure
                |         Click the Cgr Management property page
                | 
                |     The Save in CGR setting parameter represents the state of the check button
                |     named: Save FTA 3D Annotation representation in CGR.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.dmu_tol_setting_att = com_object

    @property
    def design_mode(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property DesignMode() As boolean
                | 
                |     Returns or sets the Design Mode setting parameter value.
                |     True if the Design Mode setting parameter is checked and thus set to
                |     Automatic.
                |     Role: When set to True, models are loaded in Design Mode to access
                |     technological data. Otherwise, when set to False, models are loaded in
                |     Visualization Mode, and only visualization data is loaded.

        :return: bool
        :rtype: bool
        """

        return self.dmu_tol_setting_att.DesignMode

    @design_mode.setter
    def design_mode(self, value: bool):
        """
        :param bool value:
        """

        self.dmu_tol_setting_att.DesignMode = value

    @property
    def prev_area(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property PrevArea() As boolean
                | 
                |     Returns or sets the Preview Area setting parameter value.
                |     True if the Preview Area setting parameter is checked.

        :return: bool
        :rtype: bool
        """

        return self.dmu_tol_setting_att.PrevArea

    @prev_area.setter
    def prev_area(self, value: bool):
        """
        :param bool value:
        """

        self.dmu_tol_setting_att.PrevArea = value

    @property
    def save_cgr(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property SaveCGR() As boolean
                | 
                |     Returns or sets the Save in CGR setting parameter value.
                |     True if the Save in CGR setting parameter is checked.
                |     Role: When set to True, the FTA 3D Annotation representations are saved in
                |     CGR. Otherwise, they are not saved.

        :return: bool
        :rtype: bool
        """

        return self.dmu_tol_setting_att.SaveCGR

    @save_cgr.setter
    def save_cgr(self, value: bool):
        """
        :param bool value:
        """

        self.dmu_tol_setting_att.SaveCGR = value

    @property
    def sect_pattern(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property SectPattern() As boolean
                | 
                |     Returns or sets the Pattern of Visu setting parameter
                |     value.
                |     True if the Pattern of Visu setting parameter is checked.
                |     Role: When set to True, the FTA 3D Annotation representations are saved in
                |     CGR. Otherwise, they are not saved.

        :return: bool
        :rtype: bool
        """

        return self.dmu_tol_setting_att.SectPattern

    @sect_pattern.setter
    def sect_pattern(self, value: bool):
        """
        :param bool value:
        """

        self.dmu_tol_setting_att.SectPattern = value

    def get_design_mode_info(self, admin_level: str, o_locked: str) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetDesignModeInfo(CATBSTR AdminLevel,
                | CATBSTR oLocked) As boolean
                | 
                |     Retrieves informations about the Design Mode setting parameter
                |     value.
                |     Refer to SettingController for a detailled description.

        :param str admin_level:
        :param str o_locked:
        :return: bool
        :rtype: bool
        """
        return self.dmu_tol_setting_att.GetDesignModeInfo(admin_level, o_locked)

    def get_prev_area_info(self, admin_level: str, o_locked: str) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetPrevAreaInfo(CATBSTR AdminLevel,
                | CATBSTR oLocked) As boolean
                | 
                |     Retrieves informations about the Preview Area setting parameter
                |     value.
                |     Refer to SettingController for a detailled description.

        :param str admin_level:
        :param str o_locked:
        :return: bool
        :rtype: bool
        """
        return self.dmu_tol_setting_att.GetPrevAreaInfo(admin_level, o_locked)

    def get_related_colours(self, o_related_r: int, o_related_g: int, o_related_b: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub GetRelatedColors(long oRelatedR,
                | long oRelatedG,
                | long oRelatedB)
                | 
                |     Retrieves the Related Surface Color setting parameter
                |     value.
                |     Role: The Related Surface Color setting parameter defines the color of the
                |     annotation related surfaces, that is, of all the surfaces referenced or
                |     toleranced using the CATIA V4 FD&T function.
                |     Legal values:The three color indexes range from 0 to 255.
                | 
                |     Parameters:
                | 
                |         oRelatedR
                |             [out] The Related Surface Color red index 
                |         oRelatedG
                |             [out] The Related Surface Color green index 
                |         oRelatedB
                |             [out] The Related Surface Color blue index

        :param int o_related_r:
        :param int o_related_g:
        :param int o_related_b:
        :return: None
        :rtype: None
        """
        return self.dmu_tol_setting_att.GetRelatedColors(o_related_r, o_related_g, o_related_b)

    def get_related_colours_info(self, admin_level: str, o_locked: str) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetRelatedColorsInfo(CATBSTR AdminLevel,
                | CATBSTR oLocked) As boolean
                | 
                |     Retrieves informations about the Related Surface Color setting parameter
                |     value.
                |     Refer to SettingController for a detailled description.

        :param str admin_level:
        :param str o_locked:
        :return: bool
        :rtype: bool
        """
        return self.dmu_tol_setting_att.GetRelatedColorsInfo(admin_level, o_locked)

    def get_save_cgr_info(self, admin_level: str, o_locked: str) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetSaveCGRInfo(CATBSTR AdminLevel,
                | CATBSTR oLocked) As boolean
                | 
                |     Retrieves informations about the Save in CGR setting parameter
                |     value.
                |     Refer to SettingController for a detailled description.

        :param str admin_level:
        :param str o_locked:
        :return: bool
        :rtype: bool
        """
        return self.dmu_tol_setting_att.GetSaveCGRInfo(admin_level, o_locked)

    def get_sect_pattern_info(self, admin_level: str, o_locked: str) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetSectPatternInfo(CATBSTR AdminLevel,
                | CATBSTR oLocked) As boolean
                | 
                |     Retrieves informations about the Pattern of Visu setting parameter
                |     value.
                |     Refer to SettingController for a detailled description.

        :param str admin_level:
        :param str o_locked:
        :return: bool
        :rtype: bool
        """
        return self.dmu_tol_setting_att.GetSectPatternInfo(admin_level, o_locked)

    def set_design_mode_lock(self, i_locked: bool) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetDesignModeLock(boolean iLocked)
                | 
                |     Locks or unlocks the Design Mode setting parameter value.
                |     Refer to SettingController for a detailed description.

        :param bool i_locked:
        :return: None
        :rtype: None
        """
        return self.dmu_tol_setting_att.SetDesignModeLock(i_locked)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_design_mode_lock'
        # # vba_code = """
        # # Public Function set_design_mode_lock(dmu_tol_setting_att)
        # #     Dim iLocked (2)
        # #     dmu_tol_setting_att.SetDesignModeLock iLocked
        # #     set_design_mode_lock = iLocked
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_prev_area_lock(self, i_locked: bool) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetPrevAreaLock(boolean iLocked)
                | 
                |     Locks or unlocks the Preview Area setting parameter value.
                |     Refer to SettingController for a detailed description.

        :param bool i_locked:
        :return: None
        :rtype: None
        """
        return self.dmu_tol_setting_att.SetPrevAreaLock(i_locked)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_prev_area_lock'
        # # vba_code = """
        # # Public Function set_prev_area_lock(dmu_tol_setting_att)
        # #     Dim iLocked (2)
        # #     dmu_tol_setting_att.SetPrevAreaLock iLocked
        # #     set_prev_area_lock = iLocked
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_related_colours(self, i_related_r: int, i_related_g: int, i_related_b: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetRelatedColors(long iRelatedR,
                | long iRelatedG,
                | long iRelatedB)
                | 
                |     Sets the Related Surface Color setting parameter value.
                |     Role: The Related Surface Color setting parameter defines the color of the
                |     annotation related surfaces, that is, of all the surfaces referenced or
                |     toleranced using the CATIA V4 FD&T function.
                |     Legal values:The three color indexes range from 0 to 255.
                | 
                |     Parameters:
                | 
                |         iRelatedR
                |             [in] The Related Surface Color red index 
                |         iRelatedG
                |             [in] The Related Surface Color green index 
                |         iRelatedB
                |             [in] The Related Surface Color blue index

        :param int i_related_r:
        :param int i_related_g:
        :param int i_related_b:
        :return: None
        :rtype: None
        """
        return self.dmu_tol_setting_att.SetRelatedColors(i_related_r, i_related_g, i_related_b)

    def set_related_colours_lock(self, i_locked: bool) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetRelatedColorsLock(boolean iLocked)
                | 
                |     Locks or unlocks the Related Surface Color setting parameter
                |     value.
                |     Refer to SettingController for a detailed description.

        :param bool i_locked:
        :return: None
        :rtype: None
        """
        return self.dmu_tol_setting_att.SetRelatedColorsLock(i_locked)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_related_colors_lock'
        # # vba_code = """
        # # Public Function set_related_colors_lock(dmu_tol_setting_att)
        # #     Dim iLocked (2)
        # #     dmu_tol_setting_att.SetRelatedColorsLock iLocked
        # #     set_related_colors_lock = iLocked
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_save_cgr_lock(self, i_locked: bool) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetSaveCGRLock(boolean iLocked)
                | 
                |     Locks or unlocks the Save in CGR setting parameter value.
                |     Refer to SettingController for a detailed description.

        :param bool i_locked:
        :return: None
        :rtype: None
        """
        return self.dmu_tol_setting_att.SetSaveCGRLock(i_locked)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_save_cgr_lock'
        # # vba_code = """
        # # Public Function set_save_cgr_lock(dmu_tol_setting_att)
        # #     Dim iLocked (2)
        # #     dmu_tol_setting_att.SetSaveCGRLock iLocked
        # #     set_save_cgr_lock = iLocked
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_sect_pattern_lock(self, i_locked: bool) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetSectPatternLock(boolean iLocked)
                | 
                |     Locks or unlocks the Pattern of Visu setting parameter
                |     value.
                |     Refer to SettingController for a detailed description.

        :param bool i_locked:
        :return: None
        :rtype: None
        """
        return self.dmu_tol_setting_att.SetSectPatternLock(i_locked)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_sect_pattern_lock'
        # # vba_code = """
        # # Public Function set_sect_pattern_lock(dmu_tol_setting_att)
        # #     Dim iLocked (2)
        # #     dmu_tol_setting_att.SetSectPatternLock iLocked
        # #     set_sect_pattern_lock = iLocked
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def __repr__(self):
        return f'DmuTolSettingAtt(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.in_interfaces.reference import Reference
from pycatia.knowledge_interfaces.length import Length
from pycatia.mec_mod_interfaces.hybrid_shape import HybridShape


class HybridShapeLoft(HybridShape):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.HybridShape
                |                         HybridShapeLoft
                | 
                | Represents the hybrid shape loft surface feature object.
                | Role: To access the data of the hybrid shape loft surface feature
                | object.
                | This data includes:
                | 
                |     The spine
                |     The tangent surfaces to the start and end sections
                |     Guide curves, sections, and couplings
                | 
                | Use the CATIAHybridShapeFactory to create a HybridShapeLoft
                | object.
                | 
                | LICENSING INFORMATION: Creation of volume result requires GSO
                | License
                | if GSO License is not granted , setting of Volume context has not
                | effect
                | 
                | Use the CATIAHybridShapeFactory to create a HybridShapeLoft
                | object.
                | 
                | See also:
                |     HybridShapeFactory
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.hybrid_shape_loft = com_object

    @property
    def area_law(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property AreaLaw() As Reference
                | 
                |     Gets or sets the optional area law for multi-sections element definition.
                |     The law is a length law.

        :return: Reference
        :rtype: Reference
        """

        return Reference(self.hybrid_shape_loft.AreaLaw)

    @area_law.setter
    def area_law(self, reference_law: Reference):
        """
        :param Reference reference_law:
        """

        self.hybrid_shape_loft.AreaLaw = reference_law.com_object

    @property
    def area_law_tolerance(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property AreaLawTolerance() As double
                | 
                |     Gets or sets the tolerance applied to area law. The value is in mm.

        :return: float
        :rtype: float
        """

        return self.hybrid_shape_loft.AreaLawTolerance

    @area_law_tolerance.setter
    def area_law_tolerance(self, value: float):
        """
        :param float value:
        """

        self.hybrid_shape_loft.AreaLawTolerance = value

    @property
    def boolean_operation(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property BooleanOperation() As long
                | 
                |     Gets or sets the boolean operation for closed lofted surface. TO BE USED ONLY for Part Loft
                |     (closed loft). BooleanOperation = 1 : No boolean operation. = 2 : Union boolean operation. =
                |     3 : Removal boolean operation. This example retrieves in BoolOp the type of boolean operation
                |     for the Loft hybrid shape feature.
                | 
                |      Dim BoolOp
                |      BoolOp = Loft.BooleanOperation

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_loft.BooleanOperation

    @boolean_operation.setter
    def boolean_operation(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_loft.BooleanOperation = value

    @property
    def canonical_detection(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property CanonicalDetection() As long
                | 
                |     Returns or sets whether canonical surfaces of the lofted surface are
                |     detected.
                |     Legal values:
                | 
                |     0
                |         No detection of canonical surface is performed
                |     1
                |         Detection of planar surfaces only is performed
                |     2
                |         Detection of canonical surfaces is performed

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_loft.CanonicalDetection

    @canonical_detection.setter
    def canonical_detection(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_loft.CanonicalDetection = value

    @property
    def comp_end_section_tangent(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property CompEndSectionTangent() As long
                | 
                |     Returns or sets whether the tangent surface to the end section of the
                |     lofted surface is computed.
                |     Legal values:
                | 
                |     1
                |         The tangent to the end section is computed
                |     2
                |         The tangent to the end section is not computed

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_loft.CompEndSectionTangent

    @comp_end_section_tangent.setter
    def comp_end_section_tangent(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_loft.CompEndSectionTangent = value

    @property
    def comp_start_section_tangent(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property CompStartSectionTangent() As long
                | 
                |     Returns or sets whether the tangent surface to the start section of the
                |     lofted surface is computed.
                |     Legal values:
                | 
                |     1
                |         The tangent to the start section is computed
                |     2
                |         The tangent to the start section is not computed

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_loft.CompStartSectionTangent

    @comp_start_section_tangent.setter
    def comp_start_section_tangent(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_loft.CompStartSectionTangent = value

    @property
    def context(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Context() As long
                | 
                |     Returns or sets the context on Loft feature.
                |     Legal values:
                | 
                |         0 This option creates Lofted surface.
                |         1 This option creates Lofted volume.
                | 
                | 
                |     Note: Setting volume result requires GSO License.
                | 
                |     Example:
                |         This example retrieves in oContext the context for the Loft hybrid
                |         shape feature.
                | 
                |          Dim oContext
                |          Set oContext = Loft.Context

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_loft.Context

    @context.setter
    def context(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_loft.Context = value

    @property
    def relimitation(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Relimitation() As long
                | 
                |     Returns or sets the relimitation type between sections of the
                |     loft.
                |     NOT YET IMPLEMENTED.
                |     Legal values:
                | 
                |     1
                |         The loft will be swept along the spine, then relimited by the start
                |         section and the end section
                |     2
                |         The loft will be swept along the spine.
                | 
                |             If the spine is a user spine, then the loft is limited by the spine
                |             extremities
                |             If the spine is a computed spine, then the loft is
                |             limited:
                |                 By the start section and the end section, if there is no
                |                 guide
                |                 By the guides extremities, if there are guides
                | 
                |     3
                |         The loft will be swept along the spine, then relimited by the first
                |         section,
                | 
                |             If the spine is a user spine, then the loft is limited by the spine
                |             extremity opposite to the first section
                |             If the spine is a computed spine, then the loft is
                |             limited:
                |                 By the last section, if there is no guide
                |                 By the guides extremities opposite to the first section, if
                |                 there are guides
                | 
                |     4
                |         The loft will be swept along the spine, then relimited by the last
                |         section,
                | 
                |             If the spine is a user spine, then the loft is limited by the spine
                |             extremity opposite to the last section
                |             If the spine is a computed spine, then the loft is
                |             limited:
                |                 By the first section, if there is no guide
                |                 By the guides extremities opposite to the last section, if
                |                 there are guides

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_loft.Relimitation

    @relimitation.setter
    def relimitation(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_loft.Relimitation = value

    @property
    def section_coupling(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property SectionCoupling() As long
                | 
                |     Returns or sets the type of coupling between sections of the
                |     loft.
                |     Legal values:
                | 
                |     1
                |         The curves will be coupled according to the curvilinear abscissa
                |         ratio
                |     2
                |         if each curve has the same number of tangency discontinuity points,
                |         then these points will be coupled, otherwise an error message is
                |         displayed
                |     3
                |         if each curve has the same number of tangency and curvature
                |         discontinuity points, then tangency discontinuity points will be coupled, and
                |         after curvature discontinuity points will be coupled, otherwise an error
                |         message is displayed
                |     4
                |         if each curve has the same number of vertices, then these points will
                |         be coupled, otherwise an error message is displayed

        :return: int
        :rtype: int
        """

        return self.hybrid_shape_loft.SectionCoupling

    @section_coupling.setter
    def section_coupling(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_loft.SectionCoupling = value

    @property
    def smooth_angle_threshold(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property SmoothAngleThreshold() As double
                | 
                |     Returns or sets the angular threshold under which discontinuities (moving
                |     frame, tangency net on reference surface) will be smoothed.

        :return: float
        :rtype: float
        """

        return self.hybrid_shape_loft.SmoothAngleThreshold

    @smooth_angle_threshold.setter
    def smooth_angle_threshold(self, value: float):
        """
        :param float value:
        """

        self.hybrid_shape_loft.SmoothAngleThreshold = value

    @property
    def smooth_angle_threshold_activity(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property SmoothAngleThresholdActivity() As boolean
                | 
                |     Returns or sets whether a angular threshold is allowed or not during
                |     lofting operation in order to smooth it.
                |     Legal values:
                | 
                |     TRUE
                |         The angular threshold value is used during the lofting
                |         operation
                |     FALSE
                |         The angular threshold value is not used during the lofting
                |         operation

        :return: bool
        :rtype: bool
        """

        return self.hybrid_shape_loft.SmoothAngleThresholdActivity

    @smooth_angle_threshold_activity.setter
    def smooth_angle_threshold_activity(self, value: bool):
        """
        :param bool value:
        """

        self.hybrid_shape_loft.SmoothAngleThresholdActivity = value

    @property
    def smooth_deviation(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property SmoothDeviation() As double
                | 
                |     Returns or sets the deviation value (length) allowed during lofting
                |     operation in order to smooth it.

        :return: float
        :rtype: float
        """

        return self.hybrid_shape_loft.SmoothDeviation

    @smooth_deviation.setter
    def smooth_deviation(self, value: float):
        """
        :param float value:
        """

        self.hybrid_shape_loft.SmoothDeviation = value

    @property
    def smooth_deviation_activity(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property SmoothDeviationActivity() As boolean
                | 
                |     Returns or sets whether a deviation is allowed or not during lofting
                |     operation in order to smooth it.
                |     Legal values:
                | 
                |     TRUE
                |         The deviation value is used during the lofting
                |         operation
                |     FALSE
                |         The deviation value is not used during the lofting
                |         operation

        :return: bool
        :rtype: bool
        """

        return self.hybrid_shape_loft.SmoothDeviationActivity

    @smooth_deviation_activity.setter
    def smooth_deviation_activity(self, value: bool):
        """
        :param bool value:
        """

        self.hybrid_shape_loft.SmoothDeviationActivity = value

    def add_guide(self, i_guide: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub AddGuide(Reference iGuide)
                | 
                |     Adds a guide curve to the lofted surface.
                | 
                |     Parameters:
                | 
                |         iGuide
                |             The guide curve to be added
                | 
                |             Sub-element(s) supported (see 
                | 
                |         Boundary object): TriDimFeatEdge and BiDimFeatEdge.

        :param Reference i_guide:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.AddGuide(i_guide.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'add_guide'
        # # vba_code = """
        # # Public Function add_guide(hybrid_shape_loft)
        # #     Dim iGuide (2)
        # #     hybrid_shape_loft.AddGuide iGuide
        # #     add_guide = iGuide
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def add_guide_with_tangent(self, i_guide: Reference, i_tangent: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub AddGuideWithTangent(Reference iGuide,
                | Reference iTangent)
                | 
                |     Adds a guide curve and a tangent surface to the lofted
                |     surface.
                | 
                |     Parameters:
                | 
                |         iGuide
                |             The guide curve to be added
                | 
                |             Sub-element(s) supported (see 
                | 
                |         Boundary object): TriDimFeatEdge and BiDimFeatEdge. 
                |     iTangent
                |         The tangent surface to be added. The guide curve must be layed on the
                |         tangent
                | 
                |         Sub-element(s) supported (see Boundary object): Face.

        :param Reference i_guide:
        :param Reference i_tangent:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.AddGuideWithTangent(i_guide.com_object, i_tangent.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'add_guide_with_tangent'
        # # vba_code = """
        # # Public Function add_guide_with_tangent(hybrid_shape_loft)
        # #     Dim iGuide (2)
        # #     hybrid_shape_loft.AddGuideWithTangent iGuide
        # #     add_guide_with_tangent = iGuide
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def add_section_to_loft(self, i_crv: Reference, i_ori: int, i_point: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub AddSectionToLoft(Reference iCrv,
                | long iOri,
                | Reference iPoint)
                | 
                |     Retrieves a loft section.
                | 
                |     Parameters:
                | 
                |         iCrv
                |             Reference to the curve 
                |         iOri
                |             Orientation 
                |         iPoint
                |             Reference to the Closing Point

        :param Reference i_crv:
        :param int i_ori:
        :param Reference i_point:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.AddSectionToLoft(i_crv.com_object, i_ori, i_point.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'add_section_to_loft'
        # # vba_code = """
        # # Public Function add_section_to_loft(hybrid_shape_loft)
        # #     Dim iCrv (2)
        # #     hybrid_shape_loft.AddSectionToLoft iCrv
        # #     add_section_to_loft = iCrv
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_area_law_tolerance_parameter(self) -> Length:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func GetAreaLawToleranceParameter() As Length
                | 
                |     Gets the tolerance parameter applied to area law.

        :return: Length
        :rtype: Length
        """
        return Length(self.hybrid_shape_loft.GetAreaLawToleranceParameter())

    def get_faces_for_closing(self, o_start_face: Reference, o_end_face: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetFacesForClosing(Reference oStartFace,
                | Reference oEndFace)
                | 
                |     Gets start and end faces if the tangent is a computed tangent surface to
                |     the start section or end section, from the lofted surface. The section must
                |     have been set as a face.
                | 
                |     Parameters:
                | 
                |         oStartFace
                |             start face used to close the loft. 
                |         oEndFace
                |             end face used to close the loft.

        :param Reference o_start_face:
        :param Reference o_end_face:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.GetFacesForClosing(o_start_face.com_object, o_end_face.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_faces_for_closing'
        # # vba_code = """
        # # Public Function get_faces_for_closing(hybrid_shape_loft)
        # #     Dim oStartFace (2)
        # #     hybrid_shape_loft.GetFacesForClosing oStartFace
        # #     get_faces_for_closing = oStartFace
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_guide(self, i_pos: int, o_guide: Reference, o_guide_tangent: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetGuide(long iPos,
                | Reference oGuide,
                | Reference oGuideTangent)
                | 
                |     Gets informations about the guide at a specified position in the list of
                |     the lofted surface.
                | 
                |     Parameters:
                | 
                |         iPos
                |             position of the guide in the list where the information is
                |             retrieved. 
                |         oGuide
                |             the guide curve. 
                |         oGuideTangent
                |             the tangent corresponding to the guide curve.

        :param int i_pos:
        :param Reference o_guide:
        :param Reference o_guide_tangent:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.GetGuide(i_pos, o_guide.com_object, o_guide_tangent.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_guide'
        # # vba_code = """
        # # Public Function get_guide(hybrid_shape_loft)
        # #     Dim iPos (2)
        # #     hybrid_shape_loft.GetGuide iPos
        # #     get_guide = iPos
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_nb_of_guides(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func GetNbOfGuides() As long
                | 
                |     Returns the number of guides in the loft object.
                | 
                |     Parameters:
                | 
                |         oSize
                |             Number of guides in the loft.
                | 
                |             Example:
                |                 This example retrieves the number of guides in the hybShpLoft
                |                 hybrid shape Loft.
                | 
                |                  Dim oSize As  long
                |                  oSize = hybShpLoft.GetNbOfGuides

        :return: int
        :rtype: int
        """
        return self.hybrid_shape_loft.GetNbOfGuides()

    def get_section_from_loft(self, i_rank: int, o_crv: Reference, o_ori: int, o_point: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetSectionFromLoft(long iRank,
                | Reference oCrv,
                | long oOri,
                | Reference oPoint)
                | 
                |     Retrieves a loft section information.
                | 
                |     Parameters:
                | 
                |         iRank
                |             The index of the section 
                |         oCrv
                |             The reference to the curve 
                |         oOri
                |             The orientation value 
                |         oPoint
                |             The reference to the point

        :param int i_rank:
        :param Reference o_crv:
        :param int o_ori:
        :param Reference o_point:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.GetSectionFromLoft(i_rank, o_crv.com_object, o_ori, o_point.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_section_from_loft'
        # # vba_code = """
        # # Public Function get_section_from_loft(hybrid_shape_loft)
        # #     Dim iRank (2)
        # #     hybrid_shape_loft.GetSectionFromLoft iRank
        # #     get_section_from_loft = iRank
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_spine(self, o_spine_type: int, o_spine: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetSpine(long oSpineType,
                | Reference oSpine)
                | 
                |     Gets the spine of the lofted surface.
                | 
                |     Parameters:
                | 
                |         oSpineType
                |             type of spine = 1 : User defined spine. = 2 : Automatically computed spine. 
                |         oSpine
                |             curve used as a spine, if the spine is user defined
                |             one.

        :param int o_spine_type:
        :param Reference o_spine:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.GetSpine(o_spine_type, o_spine.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_spine'
        # # vba_code = """
        # # Public Function get_spine(hybrid_shape_loft)
        # #     Dim oSpineType (2)
        # #     hybrid_shape_loft.GetSpine oSpineType
        # #     get_spine = oSpineType
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_start_and_end_section_tangent(self, o_start_section_tangent: Reference,
                                          o_end_section_tangent: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetStartAndEndSectionTangent(Reference
                | oStartSectionTangent,
                | Reference oEndSectionTangent)
                | 
                |     Gets the start and end section tangents of the lofted
                |     surface.
                | 
                |     Parameters:
                | 
                |         oStartSectionTangent
                |             tangent surface at start section. 
                |         oEndSectionTangent
                |             tangent surface at end section.

        :param Reference o_start_section_tangent:
        :param Reference o_end_section_tangent:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.GetStartAndEndSectionTangent(o_start_section_tangent.com_object,
                                                                   o_end_section_tangent.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_start_and_end_section_tangent'
        # # vba_code = """
        # # Public Function get_start_and_end_section_tangent(hybrid_shape_loft)
        # #     Dim oStartSectionTangent (2)
        # #     hybrid_shape_loft.GetStartAndEndSectionTangent oStartSectionTangent
        # #     get_start_and_end_section_tangent = oStartSectionTangent
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def insert_coupling(self, i_position: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub InsertCoupling(long iPosition)
                | 
                |     Inserts a coupling to the loft.
                | 
                |     Parameters:
                | 
                |         iPosition
                |             The position of the coupling in the list of couplings. If 0 is
                |             specified, the coupling is inserted at the end of the
                |             list.
                |             Sub-element(s) supported (see 
                | 
                |         Boundary object): Vertex.

        :param int i_position:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.InsertCoupling(i_position)

    def insert_coupling_point(self, i_coupling_index: int, i_position: int, i_point: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub InsertCouplingPoint(long iCouplingIndex,
                | long iPosition,
                | Reference iPoint)
                | 
                |     Inserts a coupling point to a coupling of the lofted
                |     surface.
                | 
                |     Parameters:
                | 
                |         iCouplingIndex
                |             The index of the coupling in the list of coupling where the point
                |             wil be inserted. 
                |         iPosition
                |             The position of the coupling point in the list of coupling points.
                |             If 0 is specified, the coupling point is inserted at the end of the list.
                |             
                |         iPoint
                |             The point to be inserted. The point must be layed on the section
                |             with the same position.
                |             Sub-element(s) supported (see 
                | 
                |         Boundary object): ScVertex.

        :param int i_coupling_index:
        :param int i_position:
        :param Reference i_point:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.InsertCouplingPoint(i_coupling_index, i_position, i_point.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'insert_coupling_point'
        # # vba_code = """
        # # Public Function insert_coupling_point(hybrid_shape_loft)
        # #     Dim iCouplingIndex (2)
        # #     hybrid_shape_loft.InsertCouplingPoint iCouplingIndex
        # #     insert_coupling_point = iCouplingIndex
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def insert_section_to_loft(self, i_type: bool, i_crv: Reference, i_ori: int, i_point: Reference,
                               i_section_ref: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub InsertSectionToLoft(boolean iType,
                | Reference iCrv,
                | long iOri,
                | Reference iPoint,
                | Reference iSectionRef)
                | 
                |     Inserts a loft section.
                | 
                |     Parameters:
                | 
                |         iType
                |             iType if set to true section is added After and iType if set to
                |             false section is added Before iSectionRef 
                |         iCrv
                |             Reference to the curve 
                |         iOri
                |             Orientation 
                |         iPoint
                |             Reference to the Closing Point 
                |         iSectionRef
                |             iSectionRef is the section before and after which section is
                |             added.

        :param bool i_type:
        :param Reference i_crv:
        :param int i_ori:
        :param Reference i_point:
        :param Reference i_section_ref:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.InsertSectionToLoft(i_type, i_crv.com_object, i_ori, i_point.com_object,
                                                          i_section_ref.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'insert_section_to_loft'
        # # vba_code = """
        # # Public Function insert_section_to_loft(hybrid_shape_loft)
        # #     Dim iType (2)
        # #     hybrid_shape_loft.InsertSectionToLoft iType
        # #     insert_section_to_loft = iType
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def modify_guide_curve(self, i_guide: Reference, i_new_guide: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub ModifyGuideCurve(Reference iGuide,
                | Reference iNewGuide)
                | 
                |     Modifies the curve of a guide from the lofted surface.
                | 
                |     Parameters:
                | 
                |         iGuide
                |             guide curve to be replaced. 
                |         iNewGuide
                |             new guide curve, will replace iGuide.

        :param Reference i_guide:
        :param Reference i_new_guide:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.ModifyGuideCurve(i_guide.com_object, i_new_guide.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'modify_guide_curve'
        # # vba_code = """
        # # Public Function modify_guide_curve(hybrid_shape_loft)
        # #     Dim iGuide (2)
        # #     hybrid_shape_loft.ModifyGuideCurve iGuide
        # #     modify_guide_curve = iGuide
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def modify_section_curve(self, i_section: Reference, i_new_section: Reference, o_curve_section: Reference,
                             o_closing_point: Reference, o_pt_diag: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub ModifySectionCurve(Reference iSection,
                | Reference iNewSection,
                | Reference oCurveSection,
                | Reference oClosingPoint,
                | long oPtDiag)
                | 
                |     Modifies the curve of section from the lofted surface.
                | 
                |     Parameters:
                | 
                |         iSection
                |             section curve to be replaced. 
                |         iNewSection
                |             section will replace iSection, can be a curve or a face
                |             
                |         oCurveSection
                |             if iSection is a face, oCurveSection is the boundary of the face.
                |             oCurveSection is used as section curve. if Part design, the face is used to
                |             close the Loft. 
                |         oClosingPoint
                |             if iSection is a closed curve, oClosingPoint is a new closing point
                |             of iSection. if iSection is a face, oClosingPoint is a new closing point the
                |             boundary of iSection. 
                |         oPtDiag
                |             Information on closing point =
                |             0 : No closing point has been created nor retrieved.
                |             1 : A closing point has been created as a vertex.
                |             2 : A closing point has been created as an extremum.
                |             3 : A closing point has been retrieved as an extremum.

        :param Reference i_section:
        :param Reference i_new_section:
        :param Reference o_curve_section:
        :param Reference o_closing_point:
        :param int o_pt_diag:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.ModifySectionCurve(i_section.com_object, i_new_section.com_object,
                                                         o_curve_section.com_object, o_closing_point.com_object,
                                                         o_pt_diag)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'modify_section_curve'
        # # vba_code = """
        # # Public Function modify_section_curve(hybrid_shape_loft)
        # #     Dim iSection (2)
        # #     hybrid_shape_loft.ModifySectionCurve iSection
        # #     modify_section_curve = iSection
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def modify_section_orient(self, i_section: Reference, i_orient: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub ModifySectionOrient(Reference iSection,
                | long iOrient)
                | 
                |     Modifies the orientation of the curve of a section from the lofted
                |     surface.
                | 
                |     Parameters:
                | 
                |         iSection
                |             section curve to be modified. 
                |         iOrient
                |             orientation of the section curve =
                |             1 : same orientation.
                |             -1 : inverted orientation.
                |             2 : ko orientation.

        :param Reference i_section:
        :param int i_orient:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.ModifySectionOrient(i_section.com_object, i_orient)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'modify_section_orient'
        # # vba_code = """
        # # Public Function modify_section_orient(hybrid_shape_loft)
        # #     Dim iSection (2)
        # #     hybrid_shape_loft.ModifySectionOrient iSection
        # #     modify_section_orient = iSection
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def remove_face_for_closing(self, i_section: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub RemoveFaceForClosing(Reference iSection)
                | 
                |     Removes face used to close the lofted surface.
                | 
                |     Parameters:
                | 
                |         iSection
                |             section curve.

        :param Reference i_section:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.RemoveFaceForClosing(i_section.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'remove_face_for_closing'
        # # vba_code = """
        # # Public Function remove_face_for_closing(hybrid_shape_loft)
        # #     Dim iSection (2)
        # #     hybrid_shape_loft.RemoveFaceForClosing iSection
        # #     remove_face_for_closing = iSection
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def remove_guide(self, i_guide: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub RemoveGuide(Reference iGuide)
                | 
                |     Removes a guide curve from the lofted surface.
                | 
                |     Parameters:
                | 
                |         iGuide
                |             The guide curve to be removed
                | 
                |             Sub-element(s) supported (see 
                | 
                |         Boundary object): TriDimFeatEdge and BiDimFeatEdge.

        :param Reference i_guide:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.RemoveGuide(i_guide.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'remove_guide'
        # # vba_code = """
        # # Public Function remove_guide(hybrid_shape_loft)
        # #     Dim iGuide (2)
        # #     hybrid_shape_loft.RemoveGuide iGuide
        # #     remove_guide = iGuide
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def remove_guide_tangent(self, i_guide: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub RemoveGuideTangent(Reference iGuide)
                | 
                |     Removes a tangent surface of a guide from the lofted
                |     surface.
                | 
                |     Parameters:
                | 
                |         iGuide
                |             guide curve of the guide from which the tangent will be
                |             removed.

        :param Reference i_guide:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.RemoveGuideTangent(i_guide.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'remove_guide_tangent'
        # # vba_code = """
        # # Public Function remove_guide_tangent(hybrid_shape_loft)
        # #     Dim iGuide (2)
        # #     hybrid_shape_loft.RemoveGuideTangent iGuide
        # #     remove_guide_tangent = iGuide
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def remove_section(self, i_section: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub RemoveSection(Reference iSection)
                | 
                |     Removes a loft section from the lofted surface.
                | 
                |     Parameters:
                | 
                |         iSection
                |             The loft section to remove
                | 
                |             Sub-element(s) supported (see 
                | 
                |         Boundary object): TriDimFeatEdge and BiDimFeatEdge.

        :param Reference i_section:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.RemoveSection(i_section.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'remove_section'
        # # vba_code = """
        # # Public Function remove_section(hybrid_shape_loft)
        # #     Dim iSection (2)
        # #     hybrid_shape_loft.RemoveSection iSection
        # #     remove_section = iSection
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def remove_section_point(self, i_section: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub RemoveSectionPoint(Reference iSection)
                | 
                |     Removes a closing point of a section from the lofted surface. The curve
                |     section must be closed curve.
                | 
                |     Parameters:
                | 
                |         iSection
                |             section curve of the section from which the point will be
                |             removed.

        :param Reference i_section:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.RemoveSectionPoint(i_section.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'remove_section_point'
        # # vba_code = """
        # # Public Function remove_section_point(hybrid_shape_loft)
        # #     Dim iSection (2)
        # #     hybrid_shape_loft.RemoveSectionPoint iSection
        # #     remove_section_point = iSection
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def remove_section_tangent(self, i_section: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub RemoveSectionTangent(Reference iSection)
                | 
                |     Removes the tangent surface of a section from the lofted surface. The
                |     section must be the start section or the end section of the
                |     loft.
                | 
                |     Parameters:
                | 
                |         iSection
                |             section curve of the section from which the tangent will be
                |             removed.

        :param Reference i_section:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.RemoveSectionTangent(i_section.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'remove_section_tangent'
        # # vba_code = """
        # # Public Function remove_section_tangent(hybrid_shape_loft)
        # #     Dim iSection (2)
        # #     hybrid_shape_loft.RemoveSectionTangent iSection
        # #     remove_section_tangent = iSection
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_end_face_for_closing(self, i_face: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetEndFaceForClosing(Reference iFace)
                | 
                |     Sets a face to the end section from the lofted surface.
                | 
                |     Parameters:
                | 
                |         iFace
                |             The face to close the loft (Part design only).
                |             Sub-element(s) supported (see 
                | 
                |         Boundary object): Face.

        :param Reference i_face:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.SetEndFaceForClosing(i_face.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_end_face_for_closing'
        # # vba_code = """
        # # Public Function set_end_face_for_closing(hybrid_shape_loft)
        # #     Dim iFace (2)
        # #     hybrid_shape_loft.SetEndFaceForClosing iFace
        # #     set_end_face_for_closing = iFace
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_end_section_tangent(self, i_tangent_section: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetEndSectionTangent(Reference iTangentSection)
                | 
                |     Sets a tangent surface to the end section from the lofted
                |     surface.
                | 
                |     Parameters:
                | 
                |         iTangentSection
                |             The tangent surface to be added. The end curve section must lay on
                |             the surface.
                |             Sub-element(s) supported (see 
                | 
                |         Boundary object): Face.

        :param Reference i_tangent_section:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.SetEndSectionTangent(i_tangent_section.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_end_section_tangent'
        # # vba_code = """
        # # Public Function set_end_section_tangent(hybrid_shape_loft)
        # #     Dim iTangentSection (2)
        # #     hybrid_shape_loft.SetEndSectionTangent iTangentSection
        # #     set_end_section_tangent = iTangentSection
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_spine(self, i_spine: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetSpine(Reference iSpine)
                | 
                |     Sets the spine to the lofted surface.
                | 
                |     Parameters:
                | 
                |         iSpine
                |             The curve to be added as a spine.
                |             Sub-element(s) supported (see 
                | 
                |         Boundary object): TriDimFeatEdge and BiDimFeatEdge.

        :param Reference i_spine:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.SetSpine(i_spine.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_spine'
        # # vba_code = """
        # # Public Function set_spine(hybrid_shape_loft)
        # #     Dim iSpine (2)
        # #     hybrid_shape_loft.SetSpine iSpine
        # #     set_spine = iSpine
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_start_face_for_closing(self, i_face: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetStartFaceForClosing(Reference iFace)
                | 
                |     Sets a face to the start section from the lofted surface.
                | 
                |     Parameters:
                | 
                |         iFace
                |             The face to close the loft (Part design only).
                |             Sub-element(s) supported (see 
                | 
                |         Boundary object): Face.

        :param Reference i_face:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.SetStartFaceForClosing(i_face.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_start_face_for_closing'
        # # vba_code = """
        # # Public Function set_start_face_for_closing(hybrid_shape_loft)
        # #     Dim iFace (2)
        # #     hybrid_shape_loft.SetStartFaceForClosing iFace
        # #     set_start_face_for_closing = iFace
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_start_section_tangent(self, i_tangent_section: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetStartSectionTangent(Reference iTangentSection)
                | 
                |     Sets a tangent surface to the start section from the lofted
                |     surface.
                | 
                |     Parameters:
                | 
                |         iTangentSection
                |             The tangent surface to be added. The start curve section must lay
                |             on the surface.
                |             Sub-element(s) supported (see 
                | 
                |         Boundary object): Face.

        :param Reference i_tangent_section:
        :return: None
        :rtype: None
        """
        return self.hybrid_shape_loft.SetStartSectionTangent(i_tangent_section.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_start_section_tangent'
        # # vba_code = """
        # # Public Function set_start_section_tangent(hybrid_shape_loft)
        # #     Dim iTangentSection (2)
        # #     hybrid_shape_loft.SetStartSectionTangent iTangentSection
        # #     set_start_section_tangent = iTangentSection
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def __repr__(self):
        return f'HybridShapeLoft(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from typing import Iterator

from pycatia.part_interfaces.draft_domain import DraftDomain
from pycatia.system_interfaces.collection import Collection
from pycatia.types.general import cat_variant


class DraftDomains(Collection):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.Collection
                |                     DraftDomains
                | 
                | The collection of draft domains used by the draft shape.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.draft_domains = com_object

    def item(self, i_index: cat_variant) -> DraftDomain:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func Item(CATVariant iIndex) As DraftDomain
                | 
                |     Returns a draft domain using its index or its name from the DraftDomains
                |     collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index or the name of the draft domain to retrieve from the
                |             collection of draft domains. As a numerics, this index is the rank of the draft
                |             domain in the collection. The index of the first draft domain in the collection
                |             is 1, and the index of the last draft domain is Count. As a string, it is the
                |             name you assigned to the draft domain using the 
                | 
                |         AnyObject.Name property. 
                |     Returns:
                |         The retrieved draft domain
                | 
                |         Example:
                |             The following example returns in domain the third draft domain of
                |             the firstDraftDomains collection:
                | 
                |              Set domain = firstDraftDomains.Item(3)

        :param cat_variant i_index:
        :return: DraftDomain
        :rtype: DraftDomain
        """
        return DraftDomain(self.draft_domains.Item(i_index))

    def __getitem__(self, n: int) -> DraftDomain:
        if (n + 1) > self.count:
            raise StopIteration

        return DraftDomain(self.draft_domains.item(n + 1))

    def __iter__(self) -> Iterator[DraftDomain]:
        for i in range(self.count):
            yield self.child_object(self.com_object.item(i + 1))

    def __repr__(self):
        return f'DraftDomains(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from typing import Iterator

from pycatia.in_interfaces.reference import Reference
from pycatia.product_structure_interfaces.product import Product
from pycatia.system_interfaces.collection import Collection
from pycatia.tps_interfaces.user_surface import UserSurface
from pycatia.types.general import cat_variant


class UserSurfaces(Collection):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.Collection
                |                     UserSurfaces

    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.user_surfaces = com_object

    def generate(self, i_support: Reference) -> UserSurface:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func Generate(Reference iSupport) As UserSurface
                | 
                |     Use this method in a Part. Creates a new user surface and adds it to the
                |     Surfaces collection.
                | 
                |     Parameters:
                | 
                |         iSupport
                |             The first reference that will support the user surface
                |             
                | 
                |     Returns:
                |         The created user surface 
                |     Example:
                |         The following example creates a user surface names NewUserSurf from the
                |         reference Ref in the Surfaces collection of the rootPart part in the partDoc
                |         part document.
                | 
                |          Set rootPart = partDoc.Part
                |          Set NewUserSurf = rootPart.UserSurfaces.Add(Ref)

        :param Reference i_support:
        :return: UserSurface
        :rtype: UserSurface
        """
        return UserSurface(self.user_surfaces.Generate(i_support.com_object))

    def generate_in_a_product_ctx(self, i_product: Product, i_prod_inst: Product, i_support: Reference) -> UserSurface:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GenerateInAProductCtx(Product iProduct,
                | Product iProdInst,
                | Reference iSupport) As UserSurface
                | 
                |     Use this method in a Product. Creates a new user surface and adds it to the
                |     Surfaces collection.
                | 
                |     Parameters:
                | 
                |         iProduct
                |             The level on which you want to create annotation (part or product).
                |             
                |         iProdInst
                |             The product instance where there is the geometry. 
                |         iSupport
                |             The first reference that will support the user surface
                |             
                | 
                |     Returns:
                |         The created user surface

        :param Product i_product:
        :param Product i_prod_inst:
        :param Reference i_support:
        :return: UserSurface
        :rtype: UserSurface
        """
        return UserSurface(self.user_surfaces.GenerateInAProductCtx(i_product.com_object, i_prod_inst.com_object,
                                                                    i_support.com_object))

    def item(self, i_index: cat_variant) -> UserSurface:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func Item(CATVariant iIndex) As UserSurface
                | 
                |     Find a user surface inside the collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The position of the users surface in the collection
                |             
                | 
                |     Returns:
                |         The user surface that is in the iIndex position in the collection

        :param CATVariant i_index:
        :return: UserSurface
        :rtype: UserSurface
        """
        return UserSurface(self.user_surfaces.Item(i_index))

    def make_user_surface_node(self, i_first_user_surf: UserSurface, i_second_user_surf: UserSurface) -> UserSurface:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func MakeUserSurfaceNode(UserSurface iFirstUserSurf,
                | UserSurface iSecondUserSurf) As UserSurface
                | 
                |     Usefull to create a User Surface Node from two others User Surface. Creates
                |     a new user surface and adds it to the Surfaces collection.
                | 
                |     Parameters:
                | 
                |         iFirstUserSurf
                |             The first User Surface to use. 
                |         iSecondUserSurf
                |             The second User Surface to use. 
                | 
                |     Returns:
                |         The created user surface

        :param UserSurface i_first_user_surf:
        :param UserSurface i_second_user_surf:
        :return: UserSurface
        :rtype: UserSurface
        """
        return UserSurface(
            self.user_surfaces.MakeUserSurfaceNode(i_first_user_surf.com_object, i_second_user_surf.com_object))

    def make_user_surface_node_2(self, i_list_of_user_surfaces: tuple) -> UserSurface:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func MakeUserSurfaceNode2(CATSafeArrayVariant iListOfUserSurfaces) As
                | UserSurface
                | 
                |     Usefull to create a User Surface Node from a list of User Surfaces. Creates
                |     a new user surface and adds it to the Surfaces collection.
                | 
                |     Parameters:
                | 
                |         iListOfUserSurfaces
                |             The list User Surfaces to use. 
                | 
                |     Returns:
                |         The created user surface

        :param tuple i_list_of_user_surfaces:
        :return: UserSurface
        :rtype: UserSurface
        """
        return UserSurface(self.user_surfaces.MakeUserSurfaceNode2(i_list_of_user_surfaces))

    def __getitem__(self, n: int) -> UserSurface:
        if (n + 1) > self.count:
            raise StopIteration

        return UserSurface(self.user_surfaces.item(n + 1))

    def __iter__(self) -> Iterator[UserSurface]:
        for i in range(self.count):
            yield self.child_object(self.com_object.item(i + 1))

    def __repr__(self):
        return f'UserSurfaces(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-09 09:53:18.676780

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.

"""
from typing import TYPE_CHECKING

from pycatia.base_interfaces.pycatia import PyCATIA

if TYPE_CHECKING:
    from pycatia.in_interfaces.application import Application


class AnyObject(PyCATIA):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-09 09:53:18.676780)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 AnyObject
                |
                | Represents the base object for all other objects except collection and
                | reference objects.
                | As a base object, it provides properties shared by any other
                | object.

    """

    def __init__(self, com_object):
        super().__init__()
        self.com_object = com_object

    @property
    def application(self) -> 'Application':
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-09 09:53:18.676780)
                | o Property Application() As Application (Read Only)
                |
                |     Returns the application. The application is the root object of the object
                |     structure and can be retrieved from any object in this object structure using
                |     the Application property. The root object, also called top-level object, is the
                |     object located at the top of the application's object structure. It is used by
                |     clients to retrieve and navigate across all the application's subordinate
                |     objects. If the client runs in-process, it retrieves the object at the top of
                |     the object structure. If the client runs out-process, it should call the
                |     GetApplication method to retrieve the object at the top of the object
                |     structure, which is the only object accessible from outside. The Application
                |     property is thus the way to jump from any object up to the root of the object
                |     structure, allowing then to navigate downwards. For in-process scripting, the
                |     application is always referred to as CATIA. Note that the Application property
                |     of the Application object returns the Application object
                |     itself.
                |
                |     Example:
                |         This example retrieves in CurrentApplication the application object,
                |         root of the object structure, from a given object of this structure: a document
                |         refered to using the MyDoc variable.
                |
                |          Dim CurrentApplication As Application
                |          Set CurrentApplication = MyDoc.Application

        :return: com_object
        :rtype: com_object
        """
        from pycatia.in_interfaces.application import Application
        return Application(self.com_object.Application)

    @property
    def name(self) -> str:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-09 09:53:18.676780)
                | o Property Name() As CATBSTR
                |
                |     Returns or sets the name of the object. The name is a character string
                |     automatically assigned to any object to handle it easier. Even if the Name
                |     property allows you to reassign an object name, this is not advised. Many
                |     objects, such as the application and the collections, have names that you must
                |     not change, and it's safer to use Name as a read only property. When an object
                |     is part of a collection, the object name can often be used in place of the
                |     object rank to retrieve or remove the object, providing the Item and Remove
                |     methods of the collection feature an argument with the Variant type. A name
                |     must start with a letter. It can include numbers, but it can't include spaces.
                |     If the object has no name set, the default name returned is the object type.
                |     For example, the Name property of a Viewer3D object with no name set returns
                |     Viewer3D.
                |
                |     Example:
                |         This example retrieves in MyObjectName the name of the MyObject
                |         object.
                |
                |          MyObjectName = MyObject.Name

        :return: str
        :rtype: str
        """

        return self.com_object.Name

    @name.setter
    def name(self, value: str):
        """
        :param str value:
        """

        self.com_object.Name = value

    @property
    def parent(self) -> 'AnyObject':
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-09 09:53:18.676780)
                | o Property Parent() As CATBaseDispatch (Read Only)
                |
                |     Returns the parent object. The parent object of a given object is the
                |     object just above in the object structure, usually the object that created this
                |     object and that aggregates it. In the case of an object part of a collection,
                |     the parent object can be the collection object itself or the object that
                |     aggregates the collection object. The Parent property is the way to step
                |     upwards in the object structure. Note that the Parent property of the
                |     Application object returns the Application object itself.
                |
                |     Example:
                |         This example retrieves in ParentObject the parent object of the
                |         GivenObject object.
                |
                |          Dim ParentObject As AnyObject
                |          Set ParentObject = GivenObject.Parent

        :return: AnyObject
        :rtype: AnyObject
        """

        return AnyObject(self.com_object.Parent)

    def get_item(self, id_name: str) -> 'AnyObject':
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-09 09:53:18.676780))
                | o Func GetItem(CATBSTR IDName) As CATBaseDispatch
                |
                |     Returns an object from its name.
                |     Role: To retrieve an object when only its name is
                |     available.
                |
                |     Parameters:
                |
                |         IDName
                |             The searched object name
                |
                |     Returns:
                |         The searched object

        :param str id_name:
        :return: AnyObject
        :rtype: AnyObject
        """
        return AnyObject(self.com_object.GetItem(id_name))

    def __repr__(self):
        return f'AnyObject(name="{self.name}")'



#! usr/bin/python3.6
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.knowledge_interfaces.int_param import IntParam
from pycatia.system_interfaces.any_object import AnyObject


class Repartition(AnyObject):

    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     Repartition
                | 
                | Represents the repartition.
                | A repartition is a set of objects used by the pattern shapes. It is the base
                | object for linear and angular repartitions.
                | 
                | See also:
                |     LinearRepartition, AngularRepartition
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.repartition = com_object

    @property
    def instances_count(self) -> IntParam:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property InstancesCount() As IntParam (Read Only)
                | 
                |     Returns the total number of copied shapes.
                | 
                |     Example:
                |         The following example returns in Nb the number of shapes of the
                |         repartition firstRepartition:
                | 
                |          Set Nb = firstRepartition.InstancesCount

        :return: IntParam
        :rtype: IntParam
        """

        return IntParam(self.repartition.InstancesCount)

    def __repr__(self):
        return f'Repartition(name="{ self.name }")'
